package builtin;

import runEnv.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

public class CounterComponent extends Component {

	public final static int STOP	= 0;
	public final static int RUN		= 1;
	public final static int STEP	= 2;

	JFrame frame;
	JTextField valueField;
	JButton startButton, stopButton, stepButton;

	int state = STOP;

	private ActionListener textInfoListener = new ActionListener() {
		public void actionPerformed( ActionEvent event ) {
			schedule();
			}
		};
	
	private ActionListener startListener = new ActionListener() {
		public void actionPerformed( ActionEvent event ) {
			int base = runEnv().getValue( "base" ).intValue();
			int n = runEnv().getValue( "n" ).intValue();
			try {
				long litValue = Long.parseLong( valueField.getText(), base );
				litValue = ( litValue + 1 ) % ( 1L << n );
				valueField.setText( Long.toString( litValue, base ) );
				}
			catch ( NumberFormatException exception ) {
				}
			state = RUN;
			schedule();
			}
		};
	
	private ActionListener stopListener = new ActionListener() {
		public void actionPerformed( ActionEvent event ) {
			state = STOP;
			}
		};
	
	private ActionListener stepListener = new ActionListener() {
		public void actionPerformed( ActionEvent event ) {
			int base = runEnv().getValue( "base" ).intValue();
			int n = runEnv().getValue( "n" ).intValue();
			try {
				long litValue = Long.parseLong( valueField.getText(), base );
				litValue = ( litValue + 1 ) % ( 1L << n );
				valueField.setText( Long.toString( litValue, base ) );
				}
			catch ( NumberFormatException exception ) {
				}
			schedule();
			}
		};
	
	public CounterComponent( RunEnv runEnv ) {
		super( runEnv );
		String name = runEnv().getValue( "name" ).stringValue();
		int x = runEnv().getValue( "x" ).intValue();
		int y = runEnv().getValue( "y" ).intValue();
		frame = new JFrame( name );

		valueField = new JTextField( 20 );
		startButton = new JButton( "Start" );
		stopButton = new JButton( "Stop" );
		stepButton = new JButton( "Step" );

		valueField.setText( "0" );

		valueField.addActionListener( textInfoListener );
		startButton.addActionListener( startListener );
		stopButton.addActionListener( stopListener );
		stepButton.addActionListener( stepListener );

		Container contentPane = frame.getContentPane();
		Box topPanel = new Box( BoxLayout.Y_AXIS );
		contentPane.add( topPanel );

		Box buttonPanel = new Box( BoxLayout.X_AXIS );

		buttonPanel.add( startButton );
		buttonPanel.add( stopButton );
		buttonPanel.add( stepButton );

		topPanel.add( valueField );
		topPanel.add( buttonPanel );
		contentPane.add( topPanel );
		
		frame.setLocation( x, y );
		frame.validate();
		frame.pack();
		frame.setVisible( true );
		frame.toFront();
		}
	
	public void compute() {
//		System.err.println( "Compute CounterComponent" );
		int base = runEnv().getValue( "base" ).intValue();
		int n = runEnv().getValue( "n" ).intValue();
		PathArrayValue result = runEnv().getValue( "result" ).pathArrayValue();
		try {
			long litValue = Long.parseLong( valueField.getText(), base );
			result.setValue( litValue % ( 1L << n ) );
			switch ( state ) {
				case STOP:
					break;
				case RUN:
					sleep( 1000 );
					litValue = ( litValue + 1 ) % ( 1L << n );
					valueField.setText( Long.toString( litValue, base ) );
					schedule();
					break;
				case STEP:
					state = STOP;
					break;
				}
			}
		catch ( NumberFormatException exception ) {
			state = STOP;
			}
		catch ( InterruptedException exception ) {
			state = STOP;
			}
		}
	
	public String toString() {
		String name = runEnv().getValue( "name" ).stringValue();
		return name;
		}
	
	}
