package node.declNode.componentDeclNode;

import node.declNode.*;
import node.declNode.pathDefnNode.*;
import node.declNode.valueDefnNode.*;

import runEnv.*;

public class ComponentDeclNode extends DeclNode {

	private PathDefnListNode inputParamDecl;
	private String ident;
	private ValueDefnListNode paramDecl;
	private PathDefnListNode outputParamDecl;
	private BodyNode body;

	public PathDefnListNode inputParamDecl() { return inputParamDecl; }
	public String ident() { return ident; }
	public ValueDefnListNode paramDecl() { return paramDecl; }
	public PathDefnListNode outputParamDecl() { return outputParamDecl; }

	public ComponentDeclNode( 
		PathDefnListNode inputParamDecl,
		String ident,
		ValueDefnListNode paramDecl,
		PathDefnListNode outputParamDecl,
		BodyNode body
		) {
		this.inputParamDecl = inputParamDecl;
		this.ident = ident;
		this.paramDecl = paramDecl;
		this.outputParamDecl = outputParamDecl;
		this.body = body;
		}

	public String toString() {
		String inputParamsText = "";
		String paramsText = "";
		String outputParamsText = "";
		if ( inputParamDecl.size() != 0 )
			inputParamsText = "{ in " + inputParamDecl + " } ";
		if ( paramDecl.size() != 0 )
			paramsText = "( " + paramDecl + " )";
		if ( outputParamDecl.size() != 0 )
			outputParamsText = " { out " + outputParamDecl + " }";
		return "component " 
			+ inputParamsText 
			+ ident 
			+ paramsText 
			+ outputParamsText 
			+ body;
		}

	public void eval( RunEnv runEnv ) {
		runEnv.declare( RunDecl.COMPONENT, ident, new ComponentValue( this, runEnv ) );
		}
	
	public void invoke( 
		RunEnv declEnv,
		PathArrayValue[] inputParamValues,
		RunValue[] paramValues,
		PathArrayValue[] outputParamValues ) {
		RunEnv localEnv = new RunEnv( ident, declEnv );
		paramDecl.eval( RunDecl.VALUEPARAM, localEnv, paramValues );
		inputParamDecl.eval( RunDecl.INPUTPARAM, localEnv, inputParamValues );
		outputParamDecl.eval( RunDecl.OUTPUTPARAM, localEnv, outputParamValues );
		body.eval( localEnv );
		}

	}
