package runEnv;

import builtin.*;
import java.util.*;

public class RunEnv {

	private String name;
	private RunEnv enclosingEnv;
	private Vector< RunDecl > locals = new Vector< RunDecl >();
	public String name() { return name; }
	private RunEnv enclosingEnv() { return enclosingEnv; }
	private int size() { return locals.size(); }
	private RunDecl elementAt( int i ) { return locals.elementAt( i ); }
	
	public RunEnv( String name, RunEnv enclosingEnv ) {
		this.name = name;
		this.enclosingEnv = enclosingEnv;
		}
	
	private RunDecl searchLocal( String ident ) {
		for ( int i = 0; i < size(); i++ ) {
			RunDecl decl = elementAt( i );
			if ( ident.equals( decl.ident() ) )
				return decl;
			}
		return null;	
		}
		
	public void declare( int kind, String ident, RunValue value ) {
		if ( searchLocal( ident ) != null )
			throw new Error( "Identifier " + ident + " redeclared" );
		locals.add( new RunDecl( kind, ident, value ) );
		}
	
	public RunValue getValue( String ident ) {
		RunDecl decl = searchLocal( ident );
		if ( decl != null )
			return decl.value();
		else if ( enclosingEnv == null )
			throw new Error( "Identifier " + ident + " undeclared" );
		else
			return enclosingEnv.getValue( ident );
		}
	
	public void addListener( Component component ) {
		for ( int i = 0; i < size(); i++ ) {
			RunDecl decl = elementAt( i );
			if ( decl.kind() == RunDecl.INPUTPARAM )
				decl.value().pathArrayValue().addListener( component );
			}
		}

	public String toString() {
		String result = name + ":\n";;
		for ( int i = 0; i < size(); i++ ) {
			RunDecl decl = elementAt( i );
			result += "\t" + decl + "\n";
			}
		return result;
		}
	}
